/*
# Solaris driver for the following USB ethernet adapters:
# Linksys USB200M, Hawking UF200, TRENDnet TU2-ET100, NetGear FA120, D-Link DUB-E100

# Copyright (C) 2003, 2004 Maxim A. Sadovski (maxim_sadovski at mail dot ru)

# This driver is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.

# This driver is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this driver; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef	_USBNET_H
#define	_USBNET_H

#ifdef  __cplusplus
extern "C" {
#endif


/*
 * Ethernet address structure
 */

struct medium_addr {
        uchar_t                 medaddr[ETHERADDRL];
};


/*
 * Definitions that go into module_info
 */

#define	USBNETIDNUM             (257)           /* module ID number */
#define	USBNETNAME              "usbnet"        /* module name */
#define	USBNETMINPSZ            (0)             /* min packet size */
#define	USBNETMAXPSZ            ETHERMAX        /* max packet size */
#define	USBNETHIWAT             (65536)         /* hi-water mark */
#define	USBNETLOWAT             (1)             /* lo-water mark */


/*
 * Per-Device instance state information.
 *
 * Each instance is dynamically allocated on first attach.
 */

struct  usbnet {
	struct usbnet           *usbnet_nextp;                  /* Next device in linked list */
	dev_info_t              *usbnet_dip;                    /* Device info */

	struct medium_addr      usbnet_ouraddr;                 /* Individual MAC address */
	struct medium_addr      usbnet_brdcstaddr;              /* Broadcast MAC address */

	uint32_t                usbnet_flags;                   /* State flags */
	kmutex_t                usbnet_xmitlock;                /* Protect transmit side */
	kmutex_t                usbnet_intrlock;                /* Protect interrupt side */
        kcondvar_t              usbnet_cond;                    /* I/O termination condition */

        gld_mac_info_t          *usbnet_mac_info;               /* GLD entry points */

        usb_client_dev_data_t   *usbnet_dev_data;               /* USB Registration Data */

        uint_t                  usb_interface;                  /* USB Interface Number */

        usb_ep_descr_t          usb_interrupt_descr;            /* Interrupt Pipe Descriptor */
        usb_ep_descr_t          usb_bulk_out_descr;             /* Bulk Out Pipe Descriptor */
        usb_ep_descr_t          usb_bulk_in_descr;              /* Bulk In Pipe Descriptor */

        usb_pipe_handle_t       usb_control_pipe;               /* Control Pipe Handle */
        usb_pipe_handle_t       usb_interrupt_pipe;             /* Interrupt Pipe Handle */
        usb_pipe_handle_t       usb_bulk_out_pipe;              /* Bulk Out Pipe Handle */
        usb_pipe_handle_t       usb_bulk_in_pipe;               /* Bulk In Pipe Handle */

        uint8_t                 usb_physical_id[2];             /* Physical IDs */
        uint8_t                 usb_link_state;                 /* State of the link */
        uint16_t                usb_temp;                       /* Used to store temporary data */

        uint8_t                 usbnet_intr_active;             /* Flag indicating that polling was initiated */

        mblk_t                  *usbnet_send_firstptr;          /* First message block of the send queue */
        mblk_t                  *usbnet_send_lastptr;           /* Last message block of the send queue */
        uint8_t                 usbnet_send_buf[ETHERMAX];      /* Buffer used to assemble sent messages */
        uint8_t                 usbnet_send_inprogress;         /* Flag indicating that send operation is in progress */
        uint8_t                 usbnet_send_active;             /* Flag indicating that send operation was initiated */

        uint8_t                 usbnet_recv_buf[ETHERMAX];      /* Buffer used to assemble received messages */
        uint8_t                 usbnet_recv_active;             /* Flag indicating that receive operation was initiated */
};


/* Our hardware USB requests and definitions */
#define USBNET_HEX_BYTES_PER_LINE       40

#define USBNET_DEFAULT_TIMEOUT          3
#define USBNET_MAX_ASYNCH_REQS          4

#define USBNET_CMD_SET_SW_MII           0x06
#define USBNET_CMD_READ_MII_REG         0x07
#define USBNET_CMD_WRITE_MII_REG        0x08
#define USBNET_CMD_SET_HW_MII           0x0a
#define USBNET_CMD_WRITE_RX_CTL         0x10
#define USBNET_CMD_READ_NODE_ID         0x17                    /* The 'Req' value to get MAC address */
#define USBNET_CMD_READ_PHY_ID          0x19                    /* The 'Req' value to get Physical ID */
#define USBNET_CMD_WRITE_MEDIUM_MODE    0x1b
#define USBNET_CMD_WRITE_GPIOS          0x1f                    /* The 'Req' value to initialize the hardware */


/* Our USB BMCR/MII defines */
#define USBNET_ADVERTISE_CSMA           0x0001                  /* Only selector supported */
#define USBNET_ADVERTISE_RESERVED       0x0400


/* Our driver states */
#define USBNET_ATTACHED                 0x01
#define USBNET_RUNNING                  0x02


/* Our link states */
#define USBNET_PHY_STATE_INITIALIZING   0x00
#define USBNET_PHY_STATE_NO_LINK        0x01
#define USBNET_PHY_STATE_POLLING_1      0x02
#define USBNET_PHY_STATE_POLLING_2      0x03
#define USBNET_PHY_STATE_POLLING_3      0x04
#define USBNET_PHY_STATE_POLLING_4      0x05
#define USBNET_PHY_STATE_SETTING_MAC    0x06
#define USBNET_PHY_STATE_LINK           0x07
#define USBNET_PHY_STATE_ABORT_POLL     0x08
#define USBNET_PHY_STATE_ABORTING       0x09


#ifdef  __cplusplus
}
#endif

#endif /* _USBNET_H */
